(defun det(A)
  "
Description:

This determinant calculator uses \"Laplace Expansion of Determinants\" to
calculate a determinant. This is a recursive algorithm that successively calls
smaller and smaller problems, until it hits a base case (1x1 matrix) and exits
with the determinant of a 1x1 matrix -- which is its only entry.

;; Examples:
 (det (matrix '(1 1) '(1)))
 (det (matrix '(2 2) '(1 0 0 1)))
 (det (matrix '(2 2) '(1 0 0 2)))
 (det (matrix '(2 2) '(1 1 2 2)))
 (det (matrix '(2 2) '(1 2 3 4)))
 (det (matrix '(3 3) '(1 2 3 4 5 6 7 8 9)))
 (det (matrix '(3 3) '(2 2 1 3 -1 0 -2 0 5)))
 ;; for the next two, see exercise 129, p. 328:
 (det (matrix '(4 4) '(1    -1     1    -1
		       1     1     0     1
		       0     1     1     0
		       0     0     1     1)))
 (det (matrix '(5 5) '(1    -1     1    -1     1
		       1     1     0     1     0
		       0     1     1     0     1
		       0     0     1     1     0
		       0     0     0     1     1)))
"
  (let* (
	 (n (first (size A)))
	 (n-1 (- n 1))
	 )
    (if (= n 1) 
	(aref A 0 0) ;; if it's 1x1, take the only element. In lisp, indexing
                     ;; starts from zero
      (sum
       (mapcar
	(lambda(m)
	  (* (aref A 0 m) ;; the mth element in the 0th (first) row. I'm lazy,
	                  ;; so I just use the 0th row. Ideally you'd choose
	                  ;; the row with the fewest non-zero elements.
	     (if (oddp m) -1 1) ;; get the sign right
	     (det  ;; now compute the determinant of a minor, by having routine
	           ;; call itself! This is called 'recursion'.
	      (cond
	       ((=  m 0) ;; the first column:
		(submatrix A 1 n-1 1 (- n 1)) 
		)
	       ((=  m n-1) ;; the last column:
		(submatrix A 1 n-1 0 (- n 2)) 
		)
	       (t ;; any other column (have to glue together two matrices). 
		(bind-columns
		 (submatrix A 1 n-1 0 (- m 1))
		 (submatrix A 1 n-1 (+ m 1) (- n 1))
		 )
		)
	       )
	      )
	     )
	  )
	(iseq n)
	)	       
       )
      )
    )
  )			 
(try-out det)
